import 'package:flutter_flicker/configuration/configuration.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'dart:math' as math;
import 'decorator_utils.dart';

class FlickerNumberInput<T> extends StatelessWidget {
  FlickerNumberInput({
    Key? key,
    required this.initialValue,
    this.onChangedInt,
    this.onChangedDouble,
    this.onChangedString,
    this.labelText,
    this.uppercaseLabelAndHint,
    this.validator,
    this.width = 250,
    this.suffix,
    this.readOnly = false,
    this.prefix,
    this.numberOfDecimals = 0,
  }) : super(key: key);

  final T initialValue;
  final String? labelText;
  final bool? uppercaseLabelAndHint;
  final ValueChanged<int>? onChangedInt;
  final ValueChanged<double>? onChangedDouble;
  final ValueChanged<String>? onChangedString;
  final FormFieldValidator<String>? validator;
  final double width;
  final Widget? suffix;
  final Widget? prefix;
  final bool readOnly;
  final int numberOfDecimals;

  @override
  Widget build(BuildContext context) {
    return TextFormField(
      enabled: !readOnly,
      readOnly: readOnly,
      style: Theme.of(context).textTheme.subtitle1?.copyWith(
            color: FlickerConfiguration.colors.input,
          ),
      decoration: _buildInputDecoration(context),
      initialValue: initialValue.toString(),
      onChanged: (value) {
        var valueStringToEvaluate = value;
        if (valueStringToEvaluate.endsWith('.')) {
          valueStringToEvaluate = valueStringToEvaluate.replaceAll('.', '');
        }

        if (numberOfDecimals == 0) {
          int? v = int.tryParse(valueStringToEvaluate);
          if (v == null) v = 0;
          if (this.onChangedInt != null) this.onChangedInt!(v);
        } else {
          double? v = double.tryParse(valueStringToEvaluate);
          if (v == null) v = 0;
          if (this.onChangedDouble != null) this.onChangedDouble!(v);
        }
        if (this.onChangedString != null) this.onChangedString!(value);
      },
      keyboardType:
          TextInputType.numberWithOptions(decimal: numberOfDecimals > 0),
      cursorColor: FlickerConfiguration.colors.inputCursor,
      inputFormatters: [
        DecimalTextInputFormatter(
          decimalRange: (numberOfDecimals == 0) ? null : numberOfDecimals,
        ),
      ],
      validator: validator,
    );
  }

  InputDecoration _buildInputDecoration(BuildContext context) {
    return InputDecoration(
      labelText: DecoratorUtils.buildLabelText(labelText),
      hintText: DecoratorUtils.buildHintText(labelText),
      prefix: prefix,
      suffix: suffix,
      labelStyle: Theme.of(context).inputDecorationTheme.labelStyle?.copyWith(
            color: readOnly
                ? Colors.grey.shade700
                : FlickerConfiguration.colors.inputLabel,
          ),
      hintStyle: Theme.of(context).inputDecorationTheme.hintStyle?.copyWith(
            color: FlickerConfiguration.colors.hintText,
          ),
      enabledBorder: Theme.of(context)
          .inputDecorationTheme
          .enabledBorder
          ?.copyWith(
            borderSide: Theme.of(context)
                .inputDecorationTheme
                .enabledBorder
                ?.borderSide
                .copyWith(
                  color: FlickerConfiguration.colors.inputControlEnabledBorder,
                ),
          ),
      focusedBorder: Theme.of(context)
          .inputDecorationTheme
          .focusedBorder
          ?.copyWith(
            borderSide: Theme.of(context)
                .inputDecorationTheme
                .focusedBorder
                ?.borderSide
                .copyWith(
                  color: FlickerConfiguration.colors.inputControlFocusedBorder,
                ),
          ),
      disabledBorder: Theme.of(context)
          .inputDecorationTheme
          .disabledBorder
          ?.copyWith(
            borderSide: Theme.of(context)
                .inputDecorationTheme
                .disabledBorder
                ?.borderSide
                .copyWith(
                  color: FlickerConfiguration.colors.inputControlDisabledBorder,
                ),
          ),
      border: Theme.of(context).inputDecorationTheme.border?.copyWith(
            borderSide: Theme.of(context)
                .inputDecorationTheme
                .border
                ?.borderSide
                .copyWith(
                  color: FlickerConfiguration.colors.inputControlBorder,
                ),
          ),
    );
  }
}

class DecimalTextInputFormatter extends TextInputFormatter {
  DecimalTextInputFormatter({this.decimalRange})
      : assert(decimalRange == null || decimalRange > 0);

  final int? decimalRange;

  @override
  TextEditingValue formatEditUpdate(
    TextEditingValue oldValue, // unused.
    TextEditingValue newValue,
  ) {
    TextSelection newSelection = newValue.selection;
    String truncated = newValue.text;

    if (decimalRange != null) {
      String value = newValue.text;

      if (value.contains(".") &&
          value.substring(value.indexOf(".") + 1).length > decimalRange!) {
        truncated = oldValue.text;
        newSelection = oldValue.selection;
      } else if (value == ".") {
        truncated = "0.";

        newSelection = newValue.selection.copyWith(
          baseOffset: math.min(truncated.length, truncated.length + 1),
          extentOffset: math.min(truncated.length, truncated.length + 1),
        );
      }

      return TextEditingValue(
        text: truncated,
        selection: newSelection,
        composing: TextRange.empty,
      );
    }
    return newValue;
  }
}
